package org.nuiton.topiatest.deletetest;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import org.apache.commons.lang.builder.ToStringBuilder;
import org.nuiton.i18n.I18n;
import org.nuiton.topia.TopiaException;
import org.nuiton.topia.persistence.EntityVisitor;
import org.nuiton.topia.persistence.TopiaEntity;
import org.nuiton.topia.persistence.TopiaEntityAbstract;
import org.nuiton.topia.persistence.util.TopiaEntityHelper;

public abstract class Party2Abstract extends TopiaEntityAbstract implements Party2 {

    /**
     * Nom de l'attribut en BD : contacts
     */
    protected Collection<Contact2> contacts;

    private static final long serialVersionUID = 3762864383461568865L;

    static {
    
        I18n.n_("topia.test.common.party2");
        I18n.n_("topia.test.common.contacts");
    }

    @Override
    public void accept(EntityVisitor visitor) throws TopiaException {
        visitor.start(this);
        visitor.visit(this, PROPERTY_CONTACTS, Collection.class, Contact2.class, contacts);
        visitor.end(this);
    }

    @Override
    public void addContacts(Contact2 contacts) {
        fireOnPreWrite(PROPERTY_CONTACTS, null, contacts);
        if (this.contacts == null) {
            this.contacts = new ArrayList<Contact2>();
        }

        if (contacts.getParty2() == null) {
            contacts.setParty2(new ArrayList<Party2>());
        }
        contacts.getParty2().add(this);

        this.contacts.add(contacts);
        fireOnPostWrite(PROPERTY_CONTACTS, this.contacts.size(), null, contacts);
    }

    @Override
    public void addAllContacts(Collection<Contact2> contacts) {
        if (contacts == null) {
            return;
        }
        for (Contact2 item : contacts) {
            addContacts(item);
        }
    }

    @Override
    public void setContacts(Collection<Contact2> contacts) {
        // Copy elements to keep data for fire with new reference
        Collection<Contact2> oldValue = this.contacts != null ? new ArrayList<Contact2>(this.contacts) : null;
        fireOnPreWrite(PROPERTY_CONTACTS, oldValue, contacts);
        this.contacts = contacts;
        fireOnPostWrite(PROPERTY_CONTACTS, oldValue, contacts);
    }

    @Override
    public void removeContacts(Contact2 contacts) {
        fireOnPreWrite(PROPERTY_CONTACTS, contacts, null);
        if (this.contacts == null || !this.contacts.remove(contacts)) {
            throw new IllegalArgumentException("List does not contain given element");
        }

        contacts.getParty2().remove(this);

        fireOnPostWrite(PROPERTY_CONTACTS, this.contacts.size() + 1, contacts, null);
    }

    @Override
    public void clearContacts() {
        if (this.contacts == null) {
            return;
        }
        for (Contact2 item : this.contacts) {
            item.getParty2().remove(this);
        }
        Collection<Contact2> oldValue = new ArrayList<Contact2>(this.contacts);
        fireOnPreWrite(PROPERTY_CONTACTS, oldValue, this.contacts);
        this.contacts.clear();
        fireOnPostWrite(PROPERTY_CONTACTS, oldValue, this.contacts);
    }

    @Override
    public Collection<Contact2> getContacts() {
        return contacts;
    }

    @Override
    public Contact2 getContactsByTopiaId(String topiaId) {
        return TopiaEntityHelper.getEntityByTopiaId(contacts, topiaId);
    }

    @Override
    public int sizeContacts() {
        if (contacts == null) {
            return 0;
        }
        return contacts.size();
    }

    @Override
    public boolean isContactsEmpty() {
        int size = sizeContacts();
        return size == 0;
    }

    @Override
    public List<TopiaEntity> getAggregate() throws TopiaException {
        List<TopiaEntity> tmp = new ArrayList<TopiaEntity>();

        // pour tous les attributs rechecher les composites et les class d'asso
        // on les ajoute dans tmp

        // on refait un tour sur chaque entity de tmp pour recuperer leur
        // composite
        List<TopiaEntity> result = new ArrayList<TopiaEntity>();
        for (TopiaEntity entity : tmp) {
            result.add(entity);
            result.addAll(entity.getAggregate());
        }
        return result;
    }

    @Override
    public List<TopiaEntity> getComposite() throws TopiaException {
        List<TopiaEntity> tmp = new ArrayList<TopiaEntity>();

        // pour tous les attributs rechecher les composites et les class d'asso
        // on les ajoute dans tmp

        // on refait un tour sur chaque entity de tmp pour recuperer leur
        // composite
        List<TopiaEntity> result = new ArrayList<TopiaEntity>();
        for (TopiaEntity entity : tmp) {
            if (entity != null) {
                result.add(entity);
                result.addAll(entity.getComposite());
            }
        }
        return result;
    }

    @Override
    public String toString() {
        String result = new ToStringBuilder(this).
         toString();
        return result;
    }

} //Party2Abstract
